/**
 * Copyright (c) 2014, 2015 Christian W. Damus and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 */
package org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator;

import java.util.ArrayList;
import java.util.Arrays;
import javax.inject.Inject;
import javax.inject.Singleton;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.ResourceLocator;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementTypeConfiguration;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementTypeSetConfiguration;
import org.eclipse.papyrus.infra.elementtypesconfigurations.SpecializationTypeConfiguration;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.ImpliedExtension;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.UMLElementTypes;
import org.eclipse.uml2.common.util.UML2Util;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.xtend.lib.annotations.Accessors;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.StringExtensions;

/**
 * Utility extensions for working with and generating unique identifiers in the element types model.
 */
@Singleton
@SuppressWarnings("all")
public class Identifiers {
  @Inject
  @Extension
  private UMLElementTypes _uMLElementTypes;
  
  @Accessors
  private String prefix;
  
  @Accessors
  private final String umlElementTypesSet = "org.eclipse.papyrus.uml.service.types.UMLElementTypeSet";
  
  @Accessors
  private final String contextId = "org.eclipse.papyrus.infra.services.edit.TypeContext";
  
  @Accessors
  private String baseElementTypesSet = this.umlElementTypesSet;
  
  @Accessors
  private ElementTypeSetConfiguration baseElementTypesSetConfiguration;
  
  @Accessors
  private boolean suppressSemanticSuperElementTypes;
  
  @Accessors
  private AdapterFactory adapterFactory;
  
  private String identifierBase;
  
  public String setIdentifierBase(final org.eclipse.uml2.uml.Package umlPackage) {
    return this.identifierBase = this.prefix;
  }
  
  public String getQualified(final String id) {
    return ((this.identifierBase + ".") + id);
  }
  
  public String toElementTypeID(final ImpliedExtension umlExtension, final ElementTypeConfiguration supertype) {
    String _xblockexpression = null;
    {
      final Stereotype stereo = umlExtension.getStereotype();
      String _xifexpression = null;
      EList<org.eclipse.uml2.uml.Class> _allExtendedMetaclasses = stereo.getAllExtendedMetaclasses();
      int _size = _allExtendedMetaclasses.size();
      boolean _lessEqualsThan = (_size <= 1);
      if (_lessEqualsThan) {
        _xifexpression = stereo.getName();
      } else {
        String _name = stereo.getName();
        String _plus = (_name + "_");
        org.eclipse.uml2.uml.Class _metaclass = umlExtension.getMetaclass();
        String _name_1 = _metaclass.getName();
        _xifexpression = (_plus + _name_1);
      }
      String name = _xifexpression;
      String _validJavaIdentifier = UML2Util.getValidJavaIdentifier(name);
      String _qualified = this.getQualified(_validJavaIdentifier);
      String _hintSuffix = this.hintSuffix(supertype);
      _xblockexpression = (_qualified + _hintSuffix);
    }
    return _xblockexpression;
  }
  
  public String toElementTypeName(final ImpliedExtension umlExtension, final ElementTypeConfiguration supertype) {
    String _xblockexpression = null;
    {
      final Stereotype stereo = umlExtension.getStereotype();
      ArrayList<Object> _newArrayList = CollectionLiterals.<Object>newArrayList();
      final Procedure1<ArrayList<Object>> _function = new Procedure1<ArrayList<Object>>() {
        public void apply(final ArrayList<Object> it) {
          EList<org.eclipse.uml2.uml.Extension> _extensions = stereo.getExtensions();
          int _size = _extensions.size();
          boolean _greaterThan = (_size > 1);
          if (_greaterThan) {
            org.eclipse.uml2.uml.Class _metaclass = umlExtension.getMetaclass();
            String _name = _metaclass.getName();
            it.add(_name);
          }
          boolean _and = false;
          String _hint = supertype.getHint();
          boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(_hint);
          boolean _not = (!_isNullOrEmpty);
          if (!_not) {
            _and = false;
          } else {
            org.eclipse.uml2.uml.Class _metaclass_1 = umlExtension.getMetaclass();
            Iterable<? extends ElementTypeConfiguration> _diagramSpecificElementTypes = Identifiers.this._uMLElementTypes.getDiagramSpecificElementTypes(_metaclass_1);
            int _size_1 = IterableExtensions.size(_diagramSpecificElementTypes);
            boolean _greaterThan_1 = (_size_1 > 1);
            _and = _greaterThan_1;
          }
          if (_and) {
            String _hint_1 = supertype.getHint();
            it.add(_hint_1);
          }
        }
      };
      final ArrayList<Object> discriminators = ObjectExtensions.<ArrayList<Object>>operator_doubleArrow(_newArrayList, _function);
      String _xifexpression = null;
      boolean _isNullOrEmpty = IterableExtensions.isNullOrEmpty(discriminators);
      if (_isNullOrEmpty) {
        String _xifexpression_1 = null;
        EList<org.eclipse.uml2.uml.Class> _allExtendedMetaclasses = stereo.getAllExtendedMetaclasses();
        int _size = _allExtendedMetaclasses.size();
        boolean _lessEqualsThan = (_size <= 1);
        if (_lessEqualsThan) {
          _xifexpression_1 = stereo.getName();
        } else {
          String _name = stereo.getName();
          String _plus = (_name + " ");
          org.eclipse.uml2.uml.Class _metaclass = umlExtension.getMetaclass();
          String _name_1 = _metaclass.getName();
          _xifexpression_1 = (_plus + _name_1);
        }
        _xifexpression = _xifexpression_1;
      } else {
        String _name_2 = stereo.getName();
        final Function1<Object, CharSequence> _function_1 = new Function1<Object, CharSequence>() {
          public CharSequence apply(final Object it) {
            return it.toString();
          }
        };
        String _join = IterableExtensions.<Object>join(discriminators, " (", ", ", ")", _function_1);
        _xifexpression = (_name_2 + _join);
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  protected String _hintSuffix(final ElementTypeConfiguration elementType) {
    return "";
  }
  
  protected String _hintSuffix(final SpecializationTypeConfiguration elementType) {
    String _xifexpression = null;
    String _hint = elementType.getHint();
    boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(_hint);
    if (_isNullOrEmpty) {
      _xifexpression = "";
    } else {
      String _hint_1 = elementType.getHint();
      _xifexpression = ("_" + _hint_1);
    }
    return _xifexpression;
  }
  
  protected String _getLabel(final EObject object) {
    String _xblockexpression = null;
    {
      Adapter _adapt = null;
      if (this.adapterFactory!=null) {
        _adapt=this.adapterFactory.adapt(object, IItemLabelProvider.class);
      }
      final IItemLabelProvider labels = ((IItemLabelProvider) _adapt);
      String _text = null;
      if (labels!=null) {
        _text=labels.getText(object);
      }
      _xblockexpression = _text;
    }
    return _xblockexpression;
  }
  
  protected String _getLabel(final EClassifier eClassifier) {
    String _xtrycatchfinallyexpression = null;
    try {
      ResourceLocator _resourceLocator = this.getResourceLocator(eClassifier);
      String _string = null;
      if (_resourceLocator!=null) {
        String _name = eClassifier.getName();
        String _plus = ("_UI_" + _name);
        String _plus_1 = (_plus + "_type");
        _string=_resourceLocator.getString(_plus_1);
      }
      _xtrycatchfinallyexpression = _string;
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception e = (Exception)_t;
        _xtrycatchfinallyexpression = eClassifier.getName();
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    return _xtrycatchfinallyexpression;
  }
  
  private ResourceLocator getResourceLocator(final EObject object) {
    Object _switchResult = null;
    Adapter _adapt = null;
    if (this.adapterFactory!=null) {
      _adapt=this.adapterFactory.adapt(object, IItemLabelProvider.class);
    }
    final Adapter adapter = _adapt;
    boolean _matched = false;
    if (!_matched) {
      if (adapter instanceof ResourceLocator) {
        _matched=true;
        _switchResult = ((Object)adapter);
      }
    }
    return ((ResourceLocator)_switchResult);
  }
  
  public String hintSuffix(final ElementTypeConfiguration elementType) {
    if (elementType instanceof SpecializationTypeConfiguration) {
      return _hintSuffix((SpecializationTypeConfiguration)elementType);
    } else if (elementType != null) {
      return _hintSuffix(elementType);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(elementType).toString());
    }
  }
  
  public String getLabel(final EObject eClassifier) {
    if (eClassifier instanceof EClassifier) {
      return _getLabel((EClassifier)eClassifier);
    } else if (eClassifier != null) {
      return _getLabel(eClassifier);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(eClassifier).toString());
    }
  }
  
  @Pure
  public String getPrefix() {
    return this.prefix;
  }
  
  public void setPrefix(final String prefix) {
    this.prefix = prefix;
  }
  
  @Pure
  public String getUmlElementTypesSet() {
    return this.umlElementTypesSet;
  }
  
  @Pure
  public String getContextId() {
    return this.contextId;
  }
  
  @Pure
  public String getBaseElementTypesSet() {
    return this.baseElementTypesSet;
  }
  
  public void setBaseElementTypesSet(final String baseElementTypesSet) {
    this.baseElementTypesSet = baseElementTypesSet;
  }
  
  @Pure
  public ElementTypeSetConfiguration getBaseElementTypesSetConfiguration() {
    return this.baseElementTypesSetConfiguration;
  }
  
  public void setBaseElementTypesSetConfiguration(final ElementTypeSetConfiguration baseElementTypesSetConfiguration) {
    this.baseElementTypesSetConfiguration = baseElementTypesSetConfiguration;
  }
  
  @Pure
  public boolean isSuppressSemanticSuperElementTypes() {
    return this.suppressSemanticSuperElementTypes;
  }
  
  public void setSuppressSemanticSuperElementTypes(final boolean suppressSemanticSuperElementTypes) {
    this.suppressSemanticSuperElementTypes = suppressSemanticSuperElementTypes;
  }
  
  @Pure
  public AdapterFactory getAdapterFactory() {
    return this.adapterFactory;
  }
  
  public void setAdapterFactory(final AdapterFactory adapterFactory) {
    this.adapterFactory = adapterFactory;
  }
}
